@@ -1,27 +1,92 @@
+import sys
+import time
+
 class Queue():
 
     # Initialize
-    def __init__(self, elements):
+    def __init__(self, elements, visualize):
+
+        start_time = time.time()
+
         self.size = len(elements)
         self.elements = elements
+        
+        # Settings
+        self.visualize = visualize
+        self.visualize_count = 0
+        self.set_maximum_size = float('INF')
+        self.set_allow_str = True
+
+        end_time = time.time()
+        time_taken = end_time - start_time
+
+        if self.visualize:
+            self.display_visualize_message("Queue has been created with the elements: " + str(elements), time_taken)
+            
 
     def __str__(self):
         return str(self.elements)
 
     def __repr__(self):
         return str(self.elements)
+
+    def display_visualize_message(self, message, time_taken):
+        self.visualize_count += 1
+        print(self.visualize_count, " - " , message)
+        for index, element in enumerate(self.elements):
+            if self.size == 1:
+                print("Front <- | ", end = "")
+                print(str(element), end = "")
+                print(" | <- Rear", end = "")
+                print()
+            elif index == 0:
+                print("Front <- | ", end = "")
+                print(str(element), end = "")
+            elif index == self.size - 1:
+                print(" | " + str(element), end = "")
+                print(" | <- Rear", end = "")
+                print()
+            else:
+                print(" | " + str(element), end = "")
+
+        print("Memory Usage: " + str(sys.getsizeof(self.elements)))
+        print("Time Taken: " + str(time_taken))
+
+    # Settings of the queue
+    def set(
+        self,
+        visualize = False,
+        maximum_size = float('INF'),
+        allow_str = True
+        ):
+        self.visualize = visualize
+        self.set_maximum_size = maximum_size
+        self.set_allow_str = allow_str
+        pass
     
     # Find the element on the index
     def index(self, position):
         if(position >= self.size or position < 0):
             print("Index out of Range")
         else:
+
+            if self.visualize:
+                self.display_visualize_message("Element at the position: " + str(position) + " is " + str(self.elements[position]))
+
             return self.elements[position]
             
     # Find the index of the element
     def find(self, element):
         try:
+
+            start_time = time.time()
             position = self.elements.index(element)
+            end_time = time.time()
+            time_taken = end_time - start_time
+
+            if self.visualize:
+                self.display_visualize_message("The element " + str(element) + " is found at the position: " + str(position), time_taken)
+
         except ValueError:
             print("Value not present.")
             position = -1
@@ -31,6 +96,19 @@ class Queue():
     # Add the element to the queue
     def add(self, position = -1, element = None):
 
+        start_time = time.time()
+
+        # CHECK SETTINGS
+        # Maximum size
+        if(self.set_maximum_size != float('INF')):
+            if(self.size >= self.set_maximum_size):
+                return "Maximum size exceeded."
+        
+        # Allow Str
+        if(not self.set_allow_str):
+            if(isinstance(element, str)):
+                return "String is not allowed."
+
         # If no element is given, but position is given, then take position as element
         if(element == None and position!=-1):
             element = position
@@ -55,6 +133,13 @@ class Queue():
                 self.elements.insert(position, element)
         self.size += 1
 
+        end_time = time.time()
+        time_taken = end_time - start_time
+
+        if self.visualize and element is not None:
+            self.display_visualize_message("The element " + str(element) + " is inserted in the position: " + str(position), time_taken)
+
+
     # Add the element in the position and remove the other element
     def add_and_remove(self, position = -1, element = None):
 
@@ -74,6 +159,7 @@ class Queue():
             if(position < self.size):
                 del self.elements[position]
             self.elements.insert(position, element)
+
         self.size += 1
 
 
@@ -89,10 +175,11 @@ class Queue():
             print("Index Out of Range")
             return -1
         else:
-            elementToReturn = self.elements[position]
+            element_to_return = self.elements[position]
             self.elements.pop(position)
             self.size -= 1
-            return elementToReturn
+
+            return element_to_return
 
     def remove_first(self):
         return self.remove(0)
@@ -157,7 +244,7 @@ class Queue():
         reversed_elements = []
         for element in self.elements[::-1]:
             reversed_elements.append(element)
-        if(inplace == True):
+        if(inplace):
             self.elements = reversed_elements
         return reversed_elements
 
@@ -178,13 +265,26 @@ class Queue():
         sorted_elements.extend(sorted_numbers)
         sorted_elements.extend(sorted_strings)
         # Convert to descending
-        if(desc == True):
+        if(desc):
             sorted_elements = list(reversed(sorted_elements))
-        if(inplace == True):
+        if(inplace):
             self.elements = sorted_elements
         return sorted_elements
 
+    # Map all the values to the function
+    def map(self, function = None, inplace = False):
+        if(not function):
+            print("Function parameter is missing.")
+            return None
+        mapped_elements = []
+        for element in self.elements:
+            value = function(element)
+            mapped_elements.append(value)
+        if(inplace):
+            self.elements = mapped_elements
+        return mapped_elements
+
 
 # CREATE A QUEUE
-def queue(elements = []):
-    return Queue(elements)
+def queue(elements = [], visualize = False):
+    return Queue(elements, visualize)
\ No newline at end of file
